import { Observable } from 'rxjs';
import { Nullable, Try } from 'javascriptutilities';
import { State as S } from 'type-safe-state-js';
import { Type as StoreType } from './types';
import { MappableObserver } from 'rx-utilities-js';
export declare namespace action {
    /**
     * Represents an action to be dispatched to the global state.
     * @template T Generics parameter.
     */
    interface Type<T> {
        readonly id: string;
        readonly fullValuePath: string;
        readonly payload: T;
    }
}
export declare type Reducer<T> = (state: S.Type<T>, action: action.Type<T>) => S.Type<T>;
/**
 * Represents a dispatch store type.
 * @extends {StoreType} Store type extension.
 */
export interface Type extends StoreType {
    dispatch(action: action.Type<any>): void;
    actionTrigger(): MappableObserver.Type<Nullable<action.Type<any>>>;
    actionStream(): Observable<action.Type<any>>;
}
/**
 * Act as a centralized storage for state. This store can dispatch actions that
 * will be reduced onto the existing state.
 * @implements {Type} Type implementation.
 */
export declare class Self implements Type {
    private readonly action;
    private readonly state;
    private readonly subscription;
    constructor();
    initialize(reducer: Reducer<any>): void;
    deinitialize(): void;
    dispatch(action: action.Type<any>): void;
    actionTrigger(): MappableObserver.Type<Nullable<action.Type<any>>>;
    actionStream(): Observable<action.Type<any>>;
    stateStream(): Observable<S.Type<any>>;
    valueAtNode(id: string): Observable<Try<any>>;
    stringAtNode(id: string): Observable<Try<string>>;
    numberAtNode(id: string): Observable<Try<number>>;
    booleanAtNode(id: string): Observable<Try<boolean>>;
}
/**
 * Create and initialize a dispatch store.
 * @param {Reducer<any>} reducer A Reducer instance.
 * @returns {Self} A dispatch store instance.
 */
export declare let createDefault: (reducer: Reducer<any>) => Self;
