"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var type_safe_state_js_1 = require("type-safe-state-js");
var Utils = require("./util");
var rx_utilities_js_1 = require("rx-utilities-js");
/**
 * Act as a centralized storage for state. This store can dispatch actions that
 * will be reduced onto the existing state.
 * @implements {Type} Type implementation.
 */
var Self = /** @class */ (function () {
    function Self() {
        this.action = new rx_utilities_js_1.IncompletableSubject(new rxjs_1.BehaviorSubject(undefined));
        this.state = new rxjs_1.BehaviorSubject(type_safe_state_js_1.State.empty());
        this.subscription = new rxjs_1.Subscription();
    }
    Self.prototype.initialize = function (reducer) {
        var disposable = this.action.asObservable()
            .pipe(rx_utilities_js_1.mapNonNilOrEmpty(function (v) { return v; }), operators_1.scan(function (acc, action) {
            return reducer(acc, action);
        }, this.state.value), operators_1.observeOn(rxjs_1.queueScheduler))
            .subscribe(this.state);
        this.subscription.add(disposable);
    };
    Self.prototype.deinitialize = function () {
        this.subscription.unsubscribe();
    };
    Self.prototype.dispatch = function (action) {
        this.actionTrigger().next(action);
    };
    Self.prototype.actionTrigger = function () {
        return this.action;
    };
    Self.prototype.actionStream = function () {
        return this.action.asObservable().pipe(rx_utilities_js_1.mapNonNilOrEmpty(function (v) { return v; }));
    };
    Self.prototype.stateStream = function () {
        return this.state;
    };
    Self.prototype.valueAtNode = function (id) {
        return Utils.valueAtNode(this.state, id);
    };
    Self.prototype.stringAtNode = function (id) {
        return Utils.stringAtNode(this.state, id);
    };
    Self.prototype.numberAtNode = function (id) {
        return Utils.numberAtNode(this.state, id);
    };
    Self.prototype.booleanAtNode = function (id) {
        return Utils.booleanAtNode(this.state, id);
    };
    return Self;
}());
exports.Self = Self;
/**
 * Create and initialize a dispatch store.
 * @param {Reducer<any>} reducer A Reducer instance.
 * @returns {Self} A dispatch store instance.
 */
exports.createDefault = function (reducer) {
    var store = new Self();
    store.initialize(reducer);
    return store;
};
//# sourceMappingURL=dispatch.js.map